import express from 'express';
import os from 'os';
import cluster from 'cluster';
import cors from 'cors';
import http from 'http'; 
import https from 'https';
import { Server } from 'socket.io';
import * as socketAdapter from "@socket.io/cluster-adapter";
import moment from 'moment';

import OtimizadorLote from './src/Services/OtimizadorLote.js';
import IntegracaoCardapio from './src/Services/IntegracaoCardapio.js';
import Status from './src/Services/Status.js';
import ExceptionHandler from './src/Exceptions/ExceptionHandler.js';
import JWT from './src/Libraries/JWT.js';
import fs from 'fs';

// const key = fs.readFileSync('./teknisa.key.nopasswd', 'utf8');
// const cert = fs.readFileSync('./teknisa-ca.crt', 'utf8');

const app = express();
// const server = http.createServer({key, cert}, app);
const server = http.createServer(app);
server.keepAliveTimeout = 32000000;
const numCPUs = os.cpus().length;
app.use(express.json());
app.use(express.urlencoded({extended: true}));
app.use((req, res, next)=>{
    res.header("Access-Control-Allow-Origin", "*");
    app.use(cors());
    next();
});
if(cluster.isMaster){
    // console.log('master -> ', process.env.HOST_ALGORITMO_SERVER);
    process.on("SIGHUP", function(){
        for(const worker of Object.values(cluster.workers)){
            worker.process.kill("SIGTERM");
        }
    });
    socketAdapter.default.setupPrimary();
    // cluster.setupMaster({
    //     serialization: "advanced",
    // });
    cluster.setupPrimary({
        serialization: "advanced",
    });
    console.log(`Master ${process.pid} is running`);
    // Fork workers.
    for (let i = 0; i < numCPUs/2; i++) {
        cluster.fork();
    }
 
    // This event is firs when worker died
    cluster.on('exit', (worker, code, signal) => {
        console.log(`worker ${worker.process.pid} died`);
    });
} else {
    process.on("SIGHUP", function(){});
    const io = new Server(server);
    io.adapter(socketAdapter.default.createAdapter());

    io.on('connection', socket => {
        console.log(`connected ${socket.id}`);
        let status = [ "FINALIZADO_COM_ERRO", "FINALIZADO_SEM_ERRO", "ERRO"];
        socket.on('message', msg => {
            const { NRITGERACAOCARD, ROW_ID } = msg;
            io.emit('onSocketSceneryBatchMessage',{OPERATION: "UPDATE_STATUS_INITIAL", NRITGERACAOCARD, ROW_ID, STATUS: 'ANDAMENTO', NRCENARIOCARD: '', TIME: 0});
        });
    });

    app.get('/', (req, resp) => {
        resp.send('<p>Hello World</p>');
    });

    app.get('/testConnection', async(req,res)=>{
        const result = await OtimizadorLote.testConnection();
        res.json(result);
    });

    app.post('/optimizer', async ( req, res ) => {
         try {
            const requestParams = req.body;
            const { NRGERACAOCARD, NRITGERACAOCARD, ROW_ID, $token } = requestParams;
            let responseStatus = await Status.updateStatusItGeracaoCard(NRITGERACAOCARD, "ANDAMENTO");
            const { CDOPERADOR, NRORG } = JWT.getTokenData($token);
            console.log(`ITEM: ${NRITGERACAOCARD} -- Start -- ${moment().format('DD/MM/YYYY HH:mm:ss')}`);
            OtimizadorLote.otimizar({
                NRGERACAOCARD, NRITGERACAOCARD, $token,
                CDOPERADOR, NRORG, ROW_ID
            }, io);
            res.json({status: 'conectado'});
        } catch ( error ) {
            ExceptionHandler.defaultHandler( error );
            res.send({error: error.message});
        }
    });

    app.post('/publish', async ( req, res ) => {
        try {
            const reqParams = req.body;
            const { CDOPERADOR, NRORG } = JWT.getTokenData(reqParams.$token);
            Object.assign(reqParams, { CDOPERADOR, NRORG });
            delete reqParams.$token;
            let responseStatus = await Status.updateStatusItGeracaoCard(reqParams.NRITGERACAOCARD, "ANDAMENTO");
            IntegracaoCardapio.integracaoCardapio(reqParams, io);
            res.json({
                status: "SUCCESS",
                data: "connected"
            });
        } catch ( error ) {
            ExceptionHandler.defaultHandler( error );
            res.send({error: error.message});
        }
    });
    
    app.post('/notifyClients', ( req, res ) => {
        const {
            EVENT_NAME,
            MESSAGE
        } = req.body;
        io.emit(EVENT_NAME, MESSAGE);
        res.json({
            status: "SUCCESS",
            data: "connected"
        });
    });

    server.listen(8080, err => {
        err ?
        console.log("Error in server...") :
        console.log(`Worker ${process.pid} started`);
    });
}


